
The VAD determines if a given signal is speech or non-speech.

The VAD is based on the energy of the signal.

The VAD is based on the signal energy.

The VAD is based on the signal spectrum.

The VAD is based on the signal spectrum.

The VAD is based on the signal spectrum.

The VAD is based on the signal spectrum.

The VAD is based on the signal spectrum.

The VAD is based on the signal spectrum.

The VAD is based on the signal spectrum.

The VAD is based on the signal spectrum.

The VAD is based on the signal spectrum.

The VAD is based on the signal spectrum.

The VAD is based on the signal spectrum.

The VAD is based on the signal spectrum.

The VAD is based on the signal spectrum.

The VAD is based on the signal spectrum.

The VAD is based on the signal spectrum.

The VAD is based on the signal spectrum.

The VAD is based on the signal spectrum.

The VAD is based on the signal spectrum.

The VAD is based on the signal spectrum.

The VAD is based on the signal spectrum.

The VAD is based on the signal spectrum.

The VAD


from transformers import AutoTokenizer, AutoModelWithLMHead, TranslationPipeline, pipelines

def translate_text(text, model="SEBIS/legal_t5_small_trans_sv_en_small_finetuned", tokenizer="SEBIS/legal_t5_small_trans_sv_en_small_finetuned"):
    pipeline = TranslationPipeline(
    model=AutoModelWithLMHead.from_pretrained(model),
    tokenizer=AutoTokenizer.from_pretrained(pretrained_model_name_or_path = tokenizer, do_lower_case=False, 
                                                skip_special_tokens=True))
    translation = pipeline(text, max_length=512)
    return translation

# sv_text = "Albert Einstein var son till Hermann och Pauline Einstein, vilka var icke-religiösa judar och tillhörde medelklassen. Fadern var försäljare och drev senare en elektroteknisk fabrik. Familjen bosatte sig 1880 i München där Einstein gick i en katolsk skola. Mängder av myter cirkulerar om Albert Einsteins person. En av dessa är att han som barn skulle ha haft svårigheter med matematik, vilket dock motsägs av hans utmärkta betyg i ämnet.[15] Han nämnde ofta senare att han inte trivdes i skolan på grund av dess pedagogik. Att Albert Einstein skulle vara släkt med musikvetaren Alfred Einstein är ett, ofta framfört, obevisat påstående. Alfred Einsteins dotter Eva har framhållit att något sådant släktskap inte existerar."

# translation = translate_text(sv_text)
# print(translation)



from pyannote.audio.pipelines import OverlappedSpeechDetection

HYPER_PARAMETERS = {
  # onset/offset activation thresholds
  "onset": 0.5, "offset": 0.5,
  # remove speech regions shorter than that many seconds.
  "min_duration_on": 0.0,
  # fill non-speech regions shorter than that many seconds.
  "min_duration_off": 0.0
}

pipeline = OverlappedSpeechDetection(segmentation="pyannote/segmentation")

def overlap_detection(audio_path):
    pipeline.instantiate(HYPER_PARAMETERS)
    overlap = pipeline(audio_path)
    # print("extracting overlap")
    print(overlap)
    return(overlap.for_json())

# file_path = "./test.wav"
# output = transcribe_from_audio_path_split_on_speech(file_path, "English")
# print(output)y7h


from transformers import pipeline

models = {
    'gpt-j': 'EleutherAI/gpt-neo-2.7B',
    "swedish-gpt": "birgermoell/swedish-gpt",
    "swe-gpt-wiki": "flax-community/swe-gpt-wiki",
    "gpt2": "huggingface/gpt2",
    "gpt2-medium": "huggingface/gpt2-medium",
    "gpt2-large": "huggingface/gpt2-large",
    "gpt2-xl": "huggingface/gpt2-xl",
    "gpt2-xlm": "huggingface/gpt2-xlm",
    "gpt2-xlm-mlm": "huggingface/gpt2-xlm-mlm",
}

def generate_text(model='birgermoell/swedish-gpt', prompt="AI har möjligheten att", min_length=50):
    generator = pipeline('text-generation', model=model)
    output = generator(prompt, do_sample=True, min_length=min_length)
    return output[0]['generated_text']

def list_models():
    return models

# output = generate_text(model='birgermoell/swedish-gpt', prompt="AI har möjligheten att", min_length=250)
# output = generate_text(model='EleutherAI/gpt-neo-2.7B', prompt="EleutherAI has", min_length=150)
# print(output)
# print(list_models())


import os
import openai
from dotenv import load_dotenv
load_dotenv()

openai.api_key = os.getenv('OPEN_AI_API_KEY')


def generate(prompt):
    response = openai.Completion.create(
    engine="davinci-codex",
    prompt=prompt,
    temperature=0,
    max_tokens=1500,
    top_p=1,
    frequency_penalty=0.5,
    presence_penalty=0.5
    )
    return response

def generate_from_prompt(text):
    response = generate(text)
    return response.choices[0].text

def write_to_file(text, file):
    info = open(file, "w")
    info.write(text)
    info.close()

response = generate_from_prompt('''
Make me a sandwich.
'''
)

write_to_file(response, "generated.py")


import torch
import torch.nn as nn
from abc import ABC, abstractmethod


class BaseSpeechModel(nn.Module, ABC):
    """
    Base class for speech model.
    """

    def __init__(self):
        super(BaseSpeechModel, self).__init__()
        self._model = None
        self._vocoder = None

    @abstractmethod
    def load_model(self, path):
        """
        Load model from path.
        """
        raise NotImplementedError("No model loaded")

    @abstractmethod
    def load_vocoder(self, path):
        """
        Load vocoder from path.
        """
        raise NotImplementedError("No vocoder loaded")

    @abstractmethod
    def synthesize(self, text, out_path):
        """
        Main method to run the synthesis
        """
        raise NotImplementedError("No synthesize implemented")

    def push_to_cpu(self, data):
        r"""
        Pushes torch.Tensor to cpu

        Args:
            data (torch.tensor): input tensor

        Returns:
            torch.tensor: tensor ensured to be on cpu
        """
        if data.device != 'cpu':
            data = data.to('cpu')
        return data



from transformers import Wav2Vec2Processor, Wav2Vec2Model
import torchaudio
import numpy as np

def get_audio_embeddings(audio_path, model_id="facebook/wav2vec2-base-960h"):
        processor = Wav2Vec2Processor.from_pretrained(model_id)
        model = Wav2Vec2Model.from_pretrained(model_id)
        waveform, sample_rate = torchaudio.load(audio_path)

        # wav_file = np.random.random((1, 1024))
        import pdb
        pdb.set_trace()

        input_values = processor(waveform, sampling_rate=sample_rate, return_tensors="pt", padding=True).input_values
        hidden_states = model(input_values).last_hidden_state
        return hidden_states

audio_embeddings = get_audio_embeddings('test.wav')


from transformers import BertTokenizer, BertModel
import torch

def get_bert_embedding_from_text(text, model="KB/bert-base-swedish-cased"):
    tokenizer = BertTokenizer.from_pretrained(model)
    model = BertModel.from_pretrained(model)
    inputs = tokenizer(text, return_tensors="pt")
    outputs = model(**inputs)
    last_hidden_states = outputs.last_hidden_state
    return last_hidden_states

# embedding = get_bert_embedding_from_text("Hej, jag gillar glass", model="KB/bert-base-swedish-cased")
# print(embedding)


from transformers import pipeline

def get_zero_shot_classification(sequence_to_classify, candidate_labels, multi_label=False):
    classifier = pipeline("zero-shot-classification",
                        model="facebook/bart-large-mnli")
    result = classifier(sequence_to_classify, candidate_labels, multi_label=multi_label)
    return result

# sequence_to_classify = "one day I will see the world"
# candidate_labels = ['travel', 'cooking', 'dancing']
# classified_label = get_zero_shot_classification(sequence_to_classify, candidate_labels)
# print(classified_label)


# from vad import extract_speak_segments
import torchaudio
import torch
import librosa
from transformers import Wav2Vec2Processor, Wav2Vec2ForCTC
import json

from pyannote.audio.pipelines import VoiceActivityDetection
# from language_files import get_model

pipeline = VoiceActivityDetection(segmentation="pyannote/segmentation")


language_dict = {
    "Swedish": "KBLab/wav2vec2-large-voxrex-swedish",
    "English": "jonatasgrosman/wav2vec2-large-xlsr-53-english",
    "Russian": "jonatasgrosman/wav2vec2-large-xlsr-53-russian",
    "Spanish": "facebook/wav2vec2-large-xlsr-53-spanish",
    "French": "facebook/wav2vec2-large-xlsr-53-french",
    "Persian": "m3hrdadfi/wav2vec2-large-xlsr-persian",
    "Dutch": "facebook/wav2vec2-large-xlsr-53-dutch",
    "Portugese": "facebook/wav2vec2-large-xlsr-53-portuguese",
    "Chinese": "jonatasgrosman/wav2vec2-large-xlsr-53-chinese-zh-cn",
    "German": "jonatasgrosman/wav2vec2-large-xlsr-53-german",
    "Greek": "lighteternal/wav2vec2-large-xlsr-53-greek",
    "Hindi": "theainerd/Wav2Vec2-large-xlsr-hindi",
    "Italian": "jonatasgrosman/wav2vec2-large-xlsr-53-italian",
    "Turkish": "cahya/wav2vec2-base-turkish-artificial-cv",
    "Vietnamese": "leduytan93/Fine-Tune-XLSR-Wav2Vec2-Speech2Text-Vietnamese",
    "Catalan": "ccoreilly/wav2vec2-large-100k-voxpopuli-catala",
    "Japanese": "vumichien/wav2vec2-large-xlsr-japanese-hiragana",
    "Tamil": "vumichien/wav2vec2-large-xlsr-japanese-hiragana",
    "Indonesian": "indonesian-nlp/wav2vec2-large-xlsr-indonesian",
    "Dhivevi": "shahukareem/wav2vec2-large-xlsr-53-dhivehi",
    "Polish": "jonatasgrosman/wav2vec2-large-xlsr-53-polish",
    "Thai": "sakares/wav2vec2-large-xlsr-thai-demo",
    "Hebrew": "imvladikon/wav2vec2-large-xlsr-53-hebrew",
    "Mongolian": "sammy786/wav2vec2-large-xlsr-mongolian",
    "Czech": "arampacha/wav2vec2-large-xlsr-czech",
    "Icelandic": "m3hrdadfi/wav2vec2-large-xlsr-icelandic",
    "Irish": "jimregan/wav2vec2-large-xlsr-irish-basic",
    "Kinyarwanda": "lucio/wav2vec2-large-xlsr-kinyarwanda",
    "Lithuanian": "DeividasM/wav2vec2-large-xlsr-53-lithuanian",
    "Hungarian": "jonatasgrosman/wav2vec2-large-xlsr-53-hungarian",
    "Finnish": "aapot/wav2vec2-large-xlsr-53-finnish",
    "Arabic": "asafaya/bert-base-arabic"
    }


HYPER_PARAMETERS = {
  # onset/offset activation thresholds
  "onset": 0.5, "offset": 0.5,
  # remove speech regions shorter than that many seconds.
  "min_duration_on": 0.0,
  # fill non-speech regions shorter than that many seconds.
  "min_duration_off": 0.0
}

def extract_speak_segments(audio_path):
    pipeline.instantiate(HYPER_PARAMETERS)
    vad = pipeline(audio_path)
    # print("extracting speaker segments")
    # print(vad)
    return(vad.for_json())

def change_sample_rate(audio_path, new_sample_rate=16000):
    audio_to_resample, sr = librosa.load(audio_path)
    resampled_audio = librosa.resample(audio_to_resample, sr, new_sample_rate)
    resampled_tensor = torch.tensor([resampled_audio])
    return resampled_tensor


def transcribe_from_audio_path_split_on_speech(audio_path, language="Swedish", model="", save_to_file="", output_format="json"):
    """
    Creates a transcription of an audio file, and outputs the
    result in one of the formats json (which is the default),
    or srt.

    If the 'save_to_file' parameter is set to a file name, the
    results will also be written to file. 

    The srt format can be used to create accurately timed subtitles 
    for videos. If 'audio_path' is the audio track of a video file, 
    a video player like VLC will create those subtitles automatically, 
    given a file in the srt format.
    """
    waveform, sample_rate = torchaudio.load(audio_path)
    if sample_rate != 16000:
        ## change sample rate to 16000
        waveform = change_sample_rate(audio_path)
        sample_rate = 16000
    segments = extract_speak_segments(audio_path)
    transcriptions = []

    try:
        model_id = language_dict[language]
    except KeyError:
        print("No language model found for %s." %language)

    if model:
        model_id = model

    processor = Wav2Vec2Processor.from_pretrained(model_id)
    model = Wav2Vec2ForCTC.from_pretrained(model_id)

    for segment in segments['content']:
        x = waveform[:,int(segment['segment']['start']*sample_rate): int(segment['segment']['end']*sample_rate)]
        with torch.no_grad():
            logits = model(x).logits
        pred_ids = torch.argmax(logits, dim=-1)
        transcription = processor.batch_decode(pred_ids)
        full_transcript = {   
            "transcription": transcription[0].encode('utf8').decode(),
            "start": segment['segment']['start'],
            "end": segment['segment']['end']
        }
        transcriptions.append(full_transcript)

    d = {}
    d['transcripts'] = transcriptions
    result = ""
    if output_format == 'json' :
        result = json.dumps(d,
                            sort_keys=False,
                            indent=4,
                            ensure_ascii=False).encode('utf8').decode()
    elif output_format == 'srt' :
        subtitle_id = 0
        for item in transcriptions :
            transcription = item['transcription']
            start = item['start']
            end = item['end']
            result += str(subtitle_id)
            result += '\n'
            result += time_format(start)
            result += ' --> '
            result += time_format(end)
            result += '\n'
            result += transcription
            result += '\n\n'
            subtitle_id += 1
    if save_to_file :
        f = open(save_to_file, "w")
        f.write( result )
        f.close()
    return result

                       
def time_format( t ) :
    """
    Produces the time format expected by the srt format,
    given a parameter t representing a number of seconds.
    """
    hours = int(t/3600)
    minutes = int((t-hours*3600)/60)
    seconds = int(t-hours*3600-minutes*60)
    fraction = int((t%1)*100)
    return str(hours) + ":" + str(minutes) + ":" + str(seconds) + "." + str(fraction)


# file_path = "./test.wav"
# output = transcribe_from_audio_path_split_on_speech(file_path, "English")
# print(output)



from transformers import pipeline

def get_answer(question_with_context):
    """
    Get answer for question and context.
    """
    # Create pipeline
    question_answering_pipeline = pipeline('question-answering')

    # Get answer
    answer = question_answering_pipeline(question_with_context)

    # Return answer
    return answer

answer = get_answer({'question': 'What is the meaning of life', 'context': 'The meaning of life is to be happy'})
print(answer)


from transformers import pipeline

# ARTICLE = """ New York (CNN)When Liana Barrientos was 23 years old, she got married in Westchester County, New York.
#  A year later, she got married again in Westchester County, but to a different man and without divorcing her first husband.
#  Only 18 days after that marriage, she got hitched yet again. Then, Barrientos declared "I do" five more times, sometimes only within two weeks of each other.
#  In 2010, she married once more, this time in the Bronx. In an application for a marriage license, she stated it was her "first and only" marriage.
#  Barrientos, now 39, is facing two criminal counts of "offering a false instrument for filing in the first degree," referring to her false statements on the
#  2010 marriage license application, according to court documents.
#  Prosecutors said the marriages were part of an immigration scam.
#  On Friday, she pleaded not guilty at State Supreme Court in the Bronx, according to her attorney, Christopher Wright, who declined to comment further.
#  After leaving court, Barrientos was arrested and charged with theft of service and criminal trespass for allegedly sneaking into the New York subway through an emergency exit, said Detective
#  Annette Markowski, a police spokeswoman. In total, Barrientos has been married 10 times, with nine of her marriages occurring between 1999 and 2002.
#  All occurred either in Westchester County, Long Island, New Jersey or the Bronx. She is believed to still be married to four men, and at one time, she was married to eight men at once, prosecutors say.
#  Prosecutors said the immigration scam involved some of her husbands, who filed for permanent residence status shortly after the marriages.
#  Any divorces happened only after such filings were approved. It was unclear whether any of the men will be prosecuted.
#  The case was referred to the Bronx District Attorney\'s Office by Immigration and Customs Enforcement and the Department of Homeland Security\'s
#  Investigation Division. Seven of the men are from so-called "red-flagged" countries, including Egypt, Turkey, Georgia, Pakistan and Mali.
#  Her eighth husband, Rashid Rajput, was deported in 2006 to his native Pakistan after an investigation by the Joint Terrorism Task Force.
#  If convicted, Barrientos faces up to four years in prison.  Her next court appearance is scheduled for May 18.
#  """

# print(summarizer(ARTICLE, max_length=130, min_length=30, do_sample=False))

def get_summary(text, model="t5-base"):
    summarizer = pipeline("summarization")
    result =summarizer(text, max_length=130, min_length=30, do_sample=False)
    return result


# sum = get_summary(ARTICLE)
# print(sum)